'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var createComponent = require('reakit-system/createComponent');
var createHook = require('reakit-system/createHook');
var Box = require('../Box/Box.js');
var _rollupPluginBabelHelpers = require('../_rollupPluginBabelHelpers-baac11a7.js');
var React = require('react');
var useAllCallbacks = require('reakit-utils/useAllCallbacks');
require('reakit-utils/useSealedState');
require('../Id/IdProvider.js');
require('../Id/IdState.js');
require('reakit-utils/applyState');
var CompositeState = require('../CompositeState-8e21e20f.js');
var getDocument = require('reakit-utils/getDocument');
var isTextField = require('reakit-utils/isTextField');
var setTextFieldValue = require('../setTextFieldValue-5b3ffb71.js');

function focusCurrentItem(widget, currentId) {
  if (currentId) {
    var _getDocument$getEleme;

    (_getDocument$getEleme = getDocument.getDocument(widget).getElementById(currentId)) === null || _getDocument$getEleme === void 0 ? void 0 : _getDocument$getEleme.focus();
  }
}

function getTextFieldValue(element) {
  if (element.isContentEditable) {
    return element.innerHTML;
  }

  return element.value;
}

var unstable_useCompositeItemWidget = createHook.createHook({
  name: "CompositeItemWidget",
  compose: Box.useBox,
  useState: CompositeState.unstable_useCompositeState,
  useProps: function useProps(options, _ref) {
    var htmlOnFocus = _ref.onFocus,
        htmlOnBlur = _ref.onBlur,
        htmlOnKeyDown = _ref.onKeyDown,
        htmlProps = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref, ["onFocus", "onBlur", "onKeyDown"]);

    var initialValue = React.useRef("");
    var onFocus = React.useCallback(function (event) {
      var _options$unstable_set;

      (_options$unstable_set = options.unstable_setHasActiveWidget) === null || _options$unstable_set === void 0 ? void 0 : _options$unstable_set.call(options, true);

      if (isTextField.isTextField(event.currentTarget)) {
        initialValue.current = getTextFieldValue(event.currentTarget);
      }
    }, [options.unstable_setHasActiveWidget]);
    var onBlur = React.useCallback(function () {
      var _options$unstable_set2;

      (_options$unstable_set2 = options.unstable_setHasActiveWidget) === null || _options$unstable_set2 === void 0 ? void 0 : _options$unstable_set2.call(options, false);
    }, [options.unstable_setHasActiveWidget]);
    var onKeyDown = React.useCallback(function (event) {
      if (event.currentTarget !== event.target) return;
      if (event.nativeEvent.isComposing) return;
      var self = event.currentTarget;

      if (event.key === "Enter") {
        if (isTextField.isTextField(self)) {
          var isMultilineTextField = self.tagName === "TEXTAREA" || self.isContentEditable; // Make sure we can create new lines using Shift+Enter

          if (isMultilineTextField && event.shiftKey) return; // Make sure it'll not trigger a click on the parent button

          event.preventDefault();
          focusCurrentItem(self, options.currentId);
        }
      } else if (event.key === "Escape") {
        focusCurrentItem(self, options.currentId);

        if (isTextField.isTextField(self)) {
          setTextFieldValue.setTextFieldValue(self, initialValue.current);
        }
      }
    }, [options.currentId]);
    return _rollupPluginBabelHelpers._objectSpread2({
      tabIndex: options.unstable_hasActiveWidget ? 0 : -1,
      onFocus: useAllCallbacks.useAllCallbacks(onFocus, htmlOnFocus),
      onBlur: useAllCallbacks.useAllCallbacks(onBlur, htmlOnBlur),
      onKeyDown: useAllCallbacks.useAllCallbacks(onKeyDown, htmlOnKeyDown),
      "data-composite-item-widget": true
    }, htmlProps);
  }
});
var unstable_CompositeItemWidget = createComponent.createComponent({
  as: "div",
  useHook: unstable_useCompositeItemWidget
});

exports.unstable_CompositeItemWidget = unstable_CompositeItemWidget;
exports.unstable_useCompositeItemWidget = unstable_useCompositeItemWidget;
