'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var createComponent = require('reakit-system/createComponent');
var createHook = require('reakit-system/createHook');
require('../Box/Box.js');
var _rollupPluginBabelHelpers = require('../_rollupPluginBabelHelpers-baac11a7.js');
var React = require('react');
var reakitWarning = require('reakit-warning');
var useAllCallbacks = require('reakit-utils/useAllCallbacks');
require('reakit-utils/useSealedState');
require('../Id/IdProvider.js');
require('../Id/IdState.js');
var IdGroup = require('../Id/IdGroup.js');
require('reakit-utils/useIsomorphicEffect');
var cx = require('reakit-utils/cx');
var DisclosureState = require('./DisclosureState.js');

function useWarningIfMultiple(options) {
  if (process.env.NODE_ENV === "production") return;
  React.useEffect(function () {
    if (!options.baseId) return;
    process.env.NODE_ENV !== "production" ? reakitWarning.warning(document.querySelectorAll("#" + options.baseId).length > 1, "You're reusing the same `useModuleState` for multiple components (DisclosureContent, Dialog, Popover, Menu etc.).", "This is not allowed! If you want to use multiple components, make sure you're using different states.", "See https://reakit.io/docs/disclosure/#multiple-components") : void 0;
  }, [options.baseId]);
}

function useSetIsMounted(options) {
  if (process.env.NODE_ENV === "production") return;
  React.useEffect(function () {
    if (options.unstable_setIsMounted) {
      options.unstable_setIsMounted(true);
    }

    return function () {
      if (options.unstable_setIsMounted) {
        options.unstable_setIsMounted(false);
      }
    };
  }, [options.unstable_setIsMounted]);
}

var useDisclosureContent = createHook.createHook({
  name: "DisclosureContent",
  compose: IdGroup.unstable_useIdGroup,
  useState: DisclosureState.useDisclosureState,
  useProps: function useProps(options, _ref) {
    var htmlOnAnimationEnd = _ref.onAnimationEnd,
        htmlOnTransitionEnd = _ref.onTransitionEnd,
        htmlClassName = _ref.className,
        htmlStyle = _ref.style,
        htmlProps = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref, ["onAnimationEnd", "onTransitionEnd", "className", "style"]);

    var _React$useState = React.useState(null),
        hiddenClass = _React$useState[0],
        setHiddenClass = _React$useState[1];

    useWarningIfMultiple(options);
    useSetIsMounted(options);
    React.useEffect(function () {
      setHiddenClass(!options.visible ? "hidden" : null);
    }, [options.visible]);
    var onTransitionEnd = React.useCallback(function (event) {
      if (event.currentTarget !== event.target) return;

      if (options.unstable_animated && options.unstable_stopAnimation) {
        options.unstable_stopAnimation();
      }
    }, [options.unstable_animated, options.unstable_stopAnimation]);
    var animating = options.unstable_animated && options.unstable_animating;
    var hidden = !options.visible && !animating;
    return _rollupPluginBabelHelpers._objectSpread2({
      id: options.baseId,
      className: cx.cx(hiddenClass, htmlClassName),
      onAnimationEnd: useAllCallbacks.useAllCallbacks(onTransitionEnd, htmlOnAnimationEnd),
      onTransitionEnd: useAllCallbacks.useAllCallbacks(onTransitionEnd, htmlOnTransitionEnd),
      hidden: hidden
    }, hidden ? {
      style: _rollupPluginBabelHelpers._objectSpread2({
        display: "none"
      }, htmlStyle)
    } : htmlStyle ? {
      style: htmlStyle
    } : {}, {}, htmlProps);
  }
});
var DisclosureContent = createComponent.createComponent({
  as: "div",
  useHook: useDisclosureContent
});

exports.DisclosureContent = DisclosureContent;
exports.useDisclosureContent = useDisclosureContent;
