'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var createComponent = require('reakit-system/createComponent');
var createHook = require('reakit-system/createHook');
require('../Box/Box.js');
var _rollupPluginBabelHelpers = require('../_rollupPluginBabelHelpers-baac11a7.js');
var React = require('react');
require('reakit-utils/useForkRef');
require('reakit-utils/isButton');
require('reakit-utils/tabbable');
require('reakit-utils/hasFocusWithin');
var reakitWarning = require('reakit-warning');
require('../Tabbable/Tabbable.js');
require('../Clickable/Clickable.js');
require('reakit-utils/useAllCallbacks');
require('reakit-utils/useSealedState');
var useCreateElement = require('reakit-system/useCreateElement');
var createOnKeyDown = require('reakit-utils/createOnKeyDown');
require('../Id/IdProvider.js');
require('../Id/IdState.js');
require('../Id/IdGroup.js');
require('reakit-utils/applyState');
require('../CompositeState-8e21e20f.js');
require('../Composite/Composite.js');
require('reakit-utils/useIsomorphicEffect');
require('../Group/Group.js');
require('../Id/Id.js');
require('../Composite/CompositeGroup.js');
require('reakit-utils/getDocument');
require('reakit-utils/isTextField');
require('reakit-utils/scrollIntoViewIfNeeded');
require('reakit-utils/useLiveRef');
require('../setTextFieldValue-5b3ffb71.js');
require('../Composite/CompositeItem.js');
require('../Composite/CompositeItemWidget.js');
require('reakit-utils/usePipe');
require('reakit-utils/cx');
require('../Disclosure/DisclosureState.js');
require('../Disclosure/DisclosureContent.js');
require('react-dom');
require('../Portal/Portal.js');
require('../Dialog/Dialog.js');
require('body-scroll-lock');
require('reakit-utils/useUpdateEffect');
require('reakit-utils/closest');
require('reakit-utils/getActiveElement');
require('reakit-utils/removeItemFromArray');
require('../Dialog/DialogState.js');
require('../DialogBackdropContext-faf4ce85.js');
require('@popperjs/core');
require('../Popover/PopoverState.js');
var Popover = require('../Popover/Popover.js');
var MenuBar = require('./MenuBar.js');
var MenuState = require('../MenuState-72cc0285.js');
require('./MenuBarState.js');

var useMenu = createHook.createHook({
  name: "Menu",
  compose: [MenuBar.useMenuBar, Popover.usePopover],
  useState: MenuState.useMenuState,
  useOptions: function useOptions(options) {
    var parent = React.useContext(MenuState.MenuContext);
    var parentIsMenuBar = parent && parent.role === "menubar";
    return _rollupPluginBabelHelpers._objectSpread2({
      unstable_autoFocusOnShow: !parent,
      unstable_autoFocusOnHide: !parentIsMenuBar,
      modal: false
    }, options, {
      // will be handled differently from usePopover/useDialog
      hideOnEsc: false
    });
  },
  useProps: function useProps(options, _ref) {
    var htmlOnKeyDown = _ref.onKeyDown,
        htmlProps = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref, ["onKeyDown"]);

    var parent = React.useContext(MenuState.MenuContext);
    var hasParent = Boolean(parent);
    var ancestorMenuBar = parent;

    while (ancestorMenuBar && ancestorMenuBar.role !== "menubar") {
      ancestorMenuBar = ancestorMenuBar.parent;
    }

    var _ref2 = ancestorMenuBar || {},
        next = _ref2.next,
        previous = _ref2.previous,
        orientation = _ref2.orientation;

    var ancestorIsHorizontal = orientation === "horizontal";

    var _split = (options.placement || "").split("-"),
        dir = _split[0];

    var onKeyDown = React.useMemo(function () {
      return createOnKeyDown.createOnKeyDown({
        onKeyDown: htmlOnKeyDown,
        stopPropagation: function stopPropagation(event) {
          // On Esc, only stop propagation if there's no parent menu.
          // Otherwise, pressing Esc should close all menus
          return event.key !== "Escape" && hasParent;
        },
        keyMap: function keyMap(_ref3) {
          var currentTarget = _ref3.currentTarget,
              target = _ref3.target;
          var Escape = options.hide;

          if (hasParent && currentTarget.contains(target)) {
            // Moves to the next menu button in a horizontal menu bar or
            // close the menu if it's a sub menu
            var ArrowRight = ancestorIsHorizontal && dir !== "left" ? next && function () {
              return next();
            } : dir === "left" && options.hide;
            var ArrowLeft = ancestorIsHorizontal && dir !== "right" ? previous && function () {
              return previous();
            } : dir === "right" && options.hide;
            return {
              Escape: Escape,
              ArrowRight: ArrowRight,
              ArrowLeft: ArrowLeft
            };
          }

          return {
            Escape: Escape
          };
        }
      });
    }, [hasParent, ancestorIsHorizontal, next, previous, dir, options.hide]);
    return _rollupPluginBabelHelpers._objectSpread2({
      role: "menu",
      onKeyDown: onKeyDown
    }, htmlProps);
  }
});
var Menu = createComponent.createComponent({
  as: "div",
  useHook: useMenu,
  useCreateElement: function useCreateElement$1(type, props, children) {
    process.env.NODE_ENV !== "production" ? reakitWarning.useWarning(!props["aria-label"] && !props["aria-labelledby"], "You should provide either `aria-label` or `aria-labelledby` props.", "See https://reakit.io/docs/menu") : void 0;
    return useCreateElement.useCreateElement(type, props, children);
  }
});

exports.Menu = Menu;
exports.useMenu = useMenu;
