'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var createComponent = require('reakit-system/createComponent');
var createHook = require('reakit-system/createHook');
require('../Box/Box.js');
var _rollupPluginBabelHelpers = require('../_rollupPluginBabelHelpers-baac11a7.js');
var React = require('react');
var useForkRef = require('reakit-utils/useForkRef');
require('reakit-utils/isButton');
require('reakit-utils/tabbable');
var hasFocusWithin = require('reakit-utils/hasFocusWithin');
require('reakit-warning');
require('../Tabbable/Tabbable.js');
require('../Clickable/Clickable.js');
require('../Button/Button.js');
var useAllCallbacks = require('reakit-utils/useAllCallbacks');
require('reakit-utils/useSealedState');
require('reakit-system/useCreateElement');
var createOnKeyDown = require('reakit-utils/createOnKeyDown');
require('../Id/IdProvider.js');
require('../Id/IdState.js');
require('../Id/IdGroup.js');
require('reakit-utils/applyState');
require('../CompositeState-8e21e20f.js');
require('../Composite/Composite.js');
require('reakit-utils/useIsomorphicEffect');
require('../Group/Group.js');
require('../Id/Id.js');
require('../Composite/CompositeGroup.js');
require('reakit-utils/getDocument');
require('reakit-utils/isTextField');
require('reakit-utils/scrollIntoViewIfNeeded');
require('reakit-utils/useLiveRef');
require('../setTextFieldValue-5b3ffb71.js');
require('../Composite/CompositeItem.js');
require('../Composite/CompositeItemWidget.js');
require('../Disclosure/DisclosureState.js');
require('reakit-utils/removeItemFromArray');
require('../Dialog/DialogState.js');
require('../Disclosure/Disclosure.js');
require('../Dialog/DialogDisclosure.js');
require('@popperjs/core');
require('../Popover/PopoverState.js');
var MenuState = require('../MenuState-72cc0285.js');
require('./MenuBarState.js');
var PopoverDisclosure = require('../Popover/PopoverDisclosure.js');

var noop = function noop() {};

var useMenuButton = createHook.createHook({
  name: "MenuButton",
  compose: PopoverDisclosure.usePopoverDisclosure,
  useState: MenuState.useMenuState,
  useProps: function useProps(options, _ref) {
    var htmlRef = _ref.ref,
        htmlOnClick = _ref.onClick,
        htmlOnKeyDown = _ref.onKeyDown,
        htmlOnFocus = _ref.onFocus,
        htmlOnMouseEnter = _ref.onMouseEnter,
        htmlOnMouseDown = _ref.onMouseDown,
        htmlProps = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref, ["ref", "onClick", "onKeyDown", "onFocus", "onMouseEnter", "onMouseDown"]);

    var parent = React.useContext(MenuState.MenuContext);
    var ref = React.useRef(null);
    var hasPressedMouse = React.useRef(false);

    var _options$placement$sp = options.placement.split("-"),
        dir = _options$placement$sp[0];

    var hasParent = Boolean(parent);
    var parentIsMenuBar = parent && parent.role === "menubar";
    var onKeyDown = React.useMemo(function () {
      return createOnKeyDown.createOnKeyDown({
        onKeyDown: htmlOnKeyDown,
        stopPropagation: function stopPropagation(event) {
          return event.key !== "Escape";
        },
        onKey: options.show,
        keyMap: function keyMap() {
          // prevents scroll jump
          var first = function first() {
            return setTimeout(options.first);
          };

          return {
            Escape: options.hide,
            Enter: hasParent && first,
            " ": hasParent && first,
            ArrowUp: dir === "top" || dir === "bottom" ? options.last : false,
            ArrowRight: dir === "right" && first,
            ArrowDown: dir === "bottom" || dir === "top" ? first : false,
            ArrowLeft: dir === "left" && first
          };
        }
      });
    }, [htmlOnKeyDown, dir, hasParent, options.show, options.hide, options.first, options.last]);
    var onMouseEnter = React.useCallback(function (event) {
      // MenuButton's don't do anything on mouse over when they aren't
      // cointained within a Menu/MenuBar
      if (!parent) return;
      var self = event.currentTarget;

      if (parentIsMenuBar) {
        var _parent$ref$current;

        // if MenuButton is an item inside a MenuBar, it'll only open
        // if there's already another sibling expanded MenuButton
        var subjacentOpenMenu = (_parent$ref$current = parent.ref.current) === null || _parent$ref$current === void 0 ? void 0 : _parent$ref$current.querySelector("[aria-expanded='true']");

        if (subjacentOpenMenu) {
          self.focus();
        }
      } else {
        // If it's in a Menu, open after a short delay
        // TODO: Make the delay a prop?
        setTimeout(function () {
          if (hasFocusWithin.hasFocusWithin(self)) {
            var _options$show;

            (_options$show = options.show) === null || _options$show === void 0 ? void 0 : _options$show.call(options);
          }
        }, 200);
      }
    }, [parent, parentIsMenuBar, options.show]);
    var onMouseDown = React.useCallback(function () {
      // When in menu bar, the menu button can be activated either by focus
      // or click, but we don't want both to trigger sequentially.
      // Otherwise, onClick would toggle (hide) the menu right after it got
      // shown on focus.
      hasPressedMouse.current = true;
    }, []);
    var onFocus = React.useCallback(function () {
      if (parentIsMenuBar && !hasPressedMouse.current) {
        var _options$show2;

        (_options$show2 = options.show) === null || _options$show2 === void 0 ? void 0 : _options$show2.call(options);
      }
    }, [parentIsMenuBar, options.show]); // If disclosure is rendered as a menu bar item, it's toggable
    // That is, you can click on the expanded disclosure to close its menu.

    var onClick = React.useCallback(function () {
      if (hasParent && !parentIsMenuBar) {
        var _options$show3;

        (_options$show3 = options.show) === null || _options$show3 === void 0 ? void 0 : _options$show3.call(options);
      } else {
        var _options$toggle;

        (_options$toggle = options.toggle) === null || _options$toggle === void 0 ? void 0 : _options$toggle.call(options);
      }

      hasPressedMouse.current = false;
    }, [hasParent, parentIsMenuBar, options.show, options.toggle]);
    return _rollupPluginBabelHelpers._objectSpread2({
      ref: useForkRef.useForkRef(ref, htmlRef),
      "aria-haspopup": "menu",
      onKeyDown: onKeyDown,
      onMouseEnter: useAllCallbacks.useAllCallbacks(onMouseEnter, htmlOnMouseEnter),
      onMouseDown: useAllCallbacks.useAllCallbacks(onMouseDown, htmlOnMouseDown),
      onFocus: useAllCallbacks.useAllCallbacks(onFocus, htmlOnFocus),
      onClick: useAllCallbacks.useAllCallbacks(onClick, htmlOnClick)
    }, htmlProps);
  },
  useComposeOptions: function useComposeOptions(options) {
    return _rollupPluginBabelHelpers._objectSpread2({}, options, {
      // Toggling is handled by MenuButton
      toggle: noop
    });
  }
});
var MenuButton = createComponent.createComponent({
  as: "button",
  useHook: useMenuButton
});

exports.MenuButton = MenuButton;
exports.useMenuButton = useMenuButton;
