'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var createComponent = require('reakit-system/createComponent');
var createHook = require('reakit-system/createHook');
require('../Box/Box.js');
var _rollupPluginBabelHelpers = require('../_rollupPluginBabelHelpers-baac11a7.js');
var React = require('react');
require('reakit-utils/useForkRef');
require('reakit-utils/isButton');
require('reakit-utils/tabbable');
require('reakit-utils/hasFocusWithin');
require('reakit-warning');
require('../Tabbable/Tabbable.js');
require('../Clickable/Clickable.js');
var useAllCallbacks = require('reakit-utils/useAllCallbacks');
require('reakit-utils/useSealedState');
require('reakit-system/useCreateElement');
require('reakit-utils/createOnKeyDown');
require('../Id/IdProvider.js');
require('../Id/IdState.js');
require('../Id/IdGroup.js');
require('reakit-utils/applyState');
require('../CompositeState-8e21e20f.js');
require('../Composite/Composite.js');
require('reakit-utils/useIsomorphicEffect');
require('../Group/Group.js');
require('../Id/Id.js');
require('../Composite/CompositeGroup.js');
var getDocument = require('reakit-utils/getDocument');
require('reakit-utils/isTextField');
require('reakit-utils/scrollIntoViewIfNeeded');
require('reakit-utils/useLiveRef');
require('../setTextFieldValue-5b3ffb71.js');
var CompositeItem = require('../Composite/CompositeItem.js');
require('../Composite/CompositeItemWidget.js');
require('../Disclosure/DisclosureState.js');
require('reakit-utils/removeItemFromArray');
require('../Dialog/DialogState.js');
require('@popperjs/core');
require('../Popover/PopoverState.js');
var MenuState = require('../MenuState-72cc0285.js');
require('./MenuBarState.js');

// TODO: Find a better implementation
function isTouchDevice() {
  if (process.env.NODE_ENV === "test" || typeof window === "undefined") return false;
  return "ontouchstart" in window || navigator.maxTouchPoints > 0 || navigator.msMaxTouchPoints > 0;
}

function isExpandedDisclosure(element) {
  return element.hasAttribute("aria-controls") && element.getAttribute("aria-expanded") === "true";
}

function getRelatedElement(event) {
  if (event.relatedTarget instanceof Element) {
    return event.relatedTarget;
  }

  return null;
}

function hoveringInside(event) {
  var self = event.currentTarget;
  var relatedElement = getRelatedElement(event);
  if (!relatedElement) return false;
  return self.contains(relatedElement);
}

function hoveringExpandedMenu(event) {
  var self = event.currentTarget;
  var relatedElement = getRelatedElement(event);
  if (!relatedElement) return false;
  var document = getDocument.getDocument(self);
  if (!isExpandedDisclosure(self)) return false;
  var menuId = self.getAttribute("aria-controls");
  var menu = document.getElementById(menuId);
  return menu === null || menu === void 0 ? void 0 : menu.contains(relatedElement);
}

function hoveringAnotherMenuItem(event, items) {
  return items === null || items === void 0 ? void 0 : items.some(function (item) {
    return item.ref.current === event.relatedTarget;
  });
}

var useMenuItem = createHook.createHook({
  name: "MenuItem",
  compose: CompositeItem.unstable_useCompositeItem,
  useState: MenuState.useMenuState,
  useProps: function useProps(options, _ref) {
    var htmlOnMouseEnter = _ref.onMouseEnter,
        htmlOnMouseLeave = _ref.onMouseLeave,
        htmlProps = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref, ["onMouseEnter", "onMouseLeave"]);

    var menu = React.useContext(MenuState.MenuContext);
    var menuRole = menu && menu.role;
    var onMouseEnter = React.useCallback(function (event) {
      if (menuRole === "menubar" || isTouchDevice()) return;
      var self = event.currentTarget;
      self.focus({
        preventScroll: true
      });
    }, [menuRole]);
    var onMouseLeave = React.useCallback(function (event) {
      var self = event.currentTarget;
      if (hoveringInside(event)) return; // If this item is a menu disclosure and mouse is leaving it to focus
      // its respective submenu, we don't want to do anything.

      if (hoveringExpandedMenu(event)) return; // On menu bars, hovering out of disclosure doesn't blur it.

      if (isExpandedDisclosure(self) && menuRole === "menubar") return; // Blur items on mouse out

      self.blur(); // Move focus to menu after blurring

      if (!hoveringAnotherMenuItem(event, options.items) && !isTouchDevice()) {
        var _options$move;

        (_options$move = options.move) === null || _options$move === void 0 ? void 0 : _options$move.call(options, null);
      }
    }, [menuRole, options.move, options.items]);
    return _rollupPluginBabelHelpers._objectSpread2({
      role: "menuitem",
      onMouseEnter: useAllCallbacks.useAllCallbacks(onMouseEnter, htmlOnMouseEnter),
      onMouseLeave: useAllCallbacks.useAllCallbacks(onMouseLeave, htmlOnMouseLeave)
    }, htmlProps);
  }
});
var MenuItem = createComponent.createComponent({
  as: "button",
  useHook: useMenuItem
});

exports.MenuItem = MenuItem;
exports.useMenuItem = useMenuItem;
