'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var createComponent = require('reakit-system/createComponent');
var createHook = require('reakit-system/createHook');
require('../Box/Box.js');
var _rollupPluginBabelHelpers = require('../_rollupPluginBabelHelpers-baac11a7.js');
var React = require('react');
var useForkRef = require('reakit-utils/useForkRef');
require('reakit-utils/isButton');
require('reakit-utils/tabbable');
require('reakit-utils/hasFocusWithin');
require('reakit-warning');
require('../Tabbable/Tabbable.js');
require('../Clickable/Clickable.js');
var useAllCallbacks = require('reakit-utils/useAllCallbacks');
require('reakit-utils/useSealedState');
require('reakit-system/useCreateElement');
require('reakit-utils/createOnKeyDown');
require('../Id/IdProvider.js');
require('../Id/IdState.js');
require('../Id/IdGroup.js');
require('reakit-utils/applyState');
require('../CompositeState-8e21e20f.js');
require('../Composite/Composite.js');
require('reakit-utils/useIsomorphicEffect');
require('../Group/Group.js');
require('../Id/Id.js');
require('../Composite/CompositeGroup.js');
require('reakit-utils/getDocument');
require('reakit-utils/isTextField');
require('reakit-utils/scrollIntoViewIfNeeded');
var useLiveRef = require('reakit-utils/useLiveRef');
require('../setTextFieldValue-5b3ffb71.js');
var CompositeItem = require('../Composite/CompositeItem.js');
require('../Composite/CompositeItemWidget.js');
var RadioState = require('./RadioState.js');

function getChecked(options) {
  if (typeof options.checked !== "undefined") {
    return options.checked;
  }

  return options.value && options.state === options.value;
}

function useInitialChecked(options) {
  var _React$useState = React.useState(function () {
    return getChecked(options);
  }),
      initialChecked = _React$useState[0];

  var _React$useState2 = React.useState(options.currentId),
      initialCurrentId = _React$useState2[0];

  var id = options.id,
      setCurrentId = options.setCurrentId;
  React.useEffect(function () {
    if (initialChecked && id && initialCurrentId !== id) {
      setCurrentId === null || setCurrentId === void 0 ? void 0 : setCurrentId(id);
    }
  }, [initialChecked, id, setCurrentId, initialCurrentId]);
}

function dispatchChange(element, onChange, originalEvent) {
  var event = originalEvent || new Event("change", {
    bubbles: true,
    cancelable: false
  });
  Object.defineProperties(event, {
    type: {
      value: "change"
    },
    target: {
      value: element
    },
    currentTarget: {
      value: element
    }
  });
  onChange === null || onChange === void 0 ? void 0 : onChange(event);
}

var useRadio = createHook.createHook({
  name: "Radio",
  compose: CompositeItem.unstable_useCompositeItem,
  useState: RadioState.useRadioState,
  keys: ["value", "checked"],
  useOptions: function useOptions(_ref, _ref2) {
    var value = _ref2.value,
        checked = _ref2.checked;

    var _ref$unstable_clickOn = _ref.unstable_clickOnEnter,
        unstable_clickOnEnter = _ref$unstable_clickOn === void 0 ? false : _ref$unstable_clickOn,
        _ref$unstable_checkOn = _ref.unstable_checkOnFocus,
        unstable_checkOnFocus = _ref$unstable_checkOn === void 0 ? true : _ref$unstable_checkOn,
        options = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref, ["unstable_clickOnEnter", "unstable_checkOnFocus"]);

    return _rollupPluginBabelHelpers._objectSpread2({
      value: value,
      checked: checked,
      unstable_clickOnEnter: unstable_clickOnEnter,
      unstable_checkOnFocus: unstable_checkOnFocus
    }, options);
  },
  useProps: function useProps(options, _ref3) {
    var htmlRef = _ref3.ref,
        htmlOnChange = _ref3.onChange,
        htmlOnClick = _ref3.onClick,
        htmlProps = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref3, ["ref", "onChange", "onClick"]);

    var ref = React.useRef(null);
    var checked = getChecked(options);
    var isCurrentItemRef = useLiveRef.useLiveRef(options.currentId === options.id);
    useInitialChecked(options);
    var onChange = React.useCallback(function (event) {
      var _options$setState;

      htmlOnChange === null || htmlOnChange === void 0 ? void 0 : htmlOnChange(event);
      if (options.disabled || event.defaultPrevented) return;
      (_options$setState = options.setState) === null || _options$setState === void 0 ? void 0 : _options$setState.call(options, options.value);
    }, [htmlOnChange, options.disabled, options.setState, options.value]);
    var onClick = React.useCallback(function (event) {
      var self = event.currentTarget;
      if (self.tagName === "INPUT") return;
      dispatchChange(self, onChange);
    }, [options.unstable_checkOnFocus, onChange]);
    React.useEffect(function () {
      var self = ref.current;
      if (!self) return;

      if (options.unstable_moves && isCurrentItemRef.current && options.unstable_checkOnFocus) {
        dispatchChange(self, onChange);
      }
    }, [options.unstable_moves, options.unstable_checkOnFocus, onChange]);
    return _rollupPluginBabelHelpers._objectSpread2({
      ref: useForkRef.useForkRef(ref, htmlRef),
      checked: checked,
      "aria-checked": checked,
      value: options.value,
      role: "radio",
      type: "radio",
      onChange: onChange,
      onClick: useAllCallbacks.useAllCallbacks(onClick, htmlOnClick)
    }, htmlProps);
  }
});
var Radio = createComponent.createComponent({
  as: "input",
  useHook: useRadio
});

exports.Radio = Radio;
exports.useRadio = useRadio;
